// Copyright 2016 Albert Nigmatzianov. All rights reserved.
// Use of this source code is governed by a MIT-style
// license that can be found in the LICENSE file.

package id3v2

import (
	"bytes"
	"io"

	"github.com/bogem/id3v2/bwpool"
	"github.com/bogem/id3v2/util"
)

// TextFrame is used to work with all text frames
// (all T*** frames like TIT2 (title), TALB (album) and so on).
//
// Example of setting a new text frame to existing tag:
//
//	textFrame := id3v2.TextFrame{
//		Encoding: id3v2.ENUTF8,
//		Text:     "Happy",
//	}
//	tag.AddFrame(tag.CommonID("Mood"), textFrame)
type TextFrame struct {
	Encoding util.Encoding
	Text     string
}

func (tf TextFrame) Size() int {
	return 1 + len(tf.Text)
}

func (tf TextFrame) WriteTo(w io.Writer) (n int64, err error) {
	var i int
	bw := bwpool.Get(w)
	defer bwpool.Put(bw)

	err = bw.WriteByte(tf.Encoding.Key)
	if err != nil {
		return
	}
	n += 1

	i, err = bw.WriteString(tf.Text)
	if err != nil {
		return
	}
	n += int64(i)

	err = bw.Flush()
	return
}

// tfBuf is used for reusing memory while parsing TextFrame.
// It is used only in parseTextFrame.
var tfBuf = new(bytes.Buffer)

func parseTextFrame(rd io.Reader) (Framer, error) {
	tfBuf.Reset()

	_, err := tfBuf.ReadFrom(rd)
	if err != nil {
		return nil, err
	}

	body := tfBuf.Bytes()
	tf := TextFrame{
		Encoding: Encodings[body[0]],
		Text:     string(body[1:]),
	}

	return tf, nil
}
